@@ -1,6 +1,6 @@
 use crate::campaign::{Campaign};
 use crate::configuration::Configuration;
-use crate::consents::{FeatureConsent, CampaignConsent, Feature};
+use crate::consents::{FeatureConsent, CampaignConsent, Feature, ConsentState, Consent};
 use crate::default_store::DefaultStore;
 use crate::event::Event;
 use crate::insights::Insights;
@@ -250,9 +250,8 @@ impl CleanInsights {
         }
 
         let feature = Vec::from_iter(features.keys())[index];
-        let consent = &features[feature];
 
-        Some(FeatureConsent::new(*feature, consent))
+        self.store.consents().consent_for_feature(*feature)
     }
 
     pub fn get_campaign_consent_by_index(&self, index: usize) -> Option<CampaignConsent> {
@@ -263,9 +262,8 @@ impl CleanInsights {
         }
 
         let campaign_id = Vec::from_iter(campaigns.keys())[index];
-        let consent = &campaigns[campaign_id];
 
-        Some(CampaignConsent::new(campaign_id.clone(), consent))
+        self.store.consents().consent_for_campaign(campaign_id)
     }
 
     pub fn grant_feature(&mut self, feature: Feature) -> FeatureConsent {
@@ -277,13 +275,39 @@ impl CleanInsights {
     }
 
     pub fn deny_feature(&mut self, feature: Feature) -> FeatureConsent {
-        let consent = self.store.consents_mut().deny(feature);
+        let consent = self.store.consents_mut().deny_feature(feature);
 
         self.persist_and_send();
 
         consent
     }
 
+    /// Returns the consent for a given feature, if any available.
+    ///
+    /// * feature: The feature to check the consent state for.
+    ///
+    /// Returns the `FeatureConsent` for the given feature or `None`, if consent unknown.
+    pub fn consent_for_feature(&self, feature: Feature) -> Option<FeatureConsent> {
+        if self.conf.server_side_anonymous_usage {
+            return Some(FeatureConsent::new(feature, &Consent::new(false)))
+        }
+
+        self.store.consents().consent_for_feature(feature)
+    }
+
+    /// Checks the consent state of a feature.
+    ///
+    /// * feature: The feature to check the consent state of.
+    ///
+    /// Returns the current state of consent.
+    pub fn state_of_feature(&self, feature: Feature) -> ConsentState {
+        if self.conf.server_side_anonymous_usage {
+            return ConsentState::Denied
+        }
+
+        self.store.consents().state_of_feature(feature)
+    }
+
     pub fn grant_campaign(&mut self, campaign_id: &str) -> Option<CampaignConsent> {
         if !self.conf.campaigns.contains_key(campaign_id) {
             return None
@@ -310,9 +334,55 @@ impl CleanInsights {
         Some(consent)
     }
 
+    /// Checks the consent state for a campaign.
+    ///
+    /// * campaign_id: The campaign ID to check the consent for.
+    ///
+    /// Returns `true`, if consent to run a campaign was given and is now valid.
     pub fn is_campaign_currently_granted(&self, campaign_id: &str) -> bool {
-        self.conf.server_side_anonymous_usage
-            || self.store.consents().is_campaign_currently_granted(campaign_id)
+        self.state_of_campaign(campaign_id) == ConsentState::Granted
+    }
+
+    /// Returns the consent for a given campaign, if any available.
+    ///
+    /// * campaign_id: The campaign ID to check the consent state for.
+    ///
+    /// Returns the `CampaignConsent` for the given campaign or `None`, if consent unknown.
+    pub fn consent_for_campaign(&self, campaign_id: &str) -> Option<CampaignConsent> {
+        if self.conf.server_side_anonymous_usage {
+            return Some(CampaignConsent::new(campaign_id.to_string(), &Consent::new(true)))
+        }
+
+        if !self.conf.campaigns.contains_key(campaign_id) {
+            return None
+        }
+
+        if Utc::now() >= self.conf.campaigns[campaign_id].end {
+            return None
+        }
+
+        self.store.consents().consent_for_campaign(campaign_id)
+    }
+
+    /// Checks the consent state of a campaign.
+    ///
+    /// * campaign_id: The campaign ID to check the consent state of.
+    ///
+    /// Returns the current state of consent.
+    pub fn state_of_campaign(&self, campaign_id: &str) -> ConsentState {
+        if self.conf.server_side_anonymous_usage {
+            return ConsentState::Granted
+        }
+
+        if !self.conf.campaigns.contains_key(campaign_id) {
+            return ConsentState::Unconfigured
+        }
+
+        if Utc::now() >= self.conf.campaigns[campaign_id].end {
+            return ConsentState::Unconfigured
+        }
+
+        self.store.consents().state_of_campaign(campaign_id)
     }
 
     pub fn can_ask_consent_for_campaign(&self, campaign_id: &str) -> Option<(DateTime<Utc>, DateTime<Utc>)> {
